// $Id: DOMCharacterData.m,v 1.2 2003/01/02 09:51:24 helge Exp $

#include "DOMCharacterData.h"
#include "common.h"

@implementation DOMCharacterData

- (id)initWithString:(NSString *)_s {
  if ((self = [super init])) {
    self->data = [_s copy];
  }
  return self;
}
- (void)dealloc {
  [self->data release];
  [super dealloc];
}

/* attributes */

- (void)setData:(NSString *)_data {
  id old = self->data;
  self->data = [_data copy];
  [old release];
}
- (NSString *)data {
  return self->data;
}

- (unsigned)length {
  return [self->data length];
}

/* operations */

- (NSString *)substringData:(unsigned)_offset count:(unsigned)_count {
  NSRange r;

  r.location = _offset;
  r.length   = _count;
  
  return [[self data] substringWithRange:r];
}

- (void)appendData:(NSString *)_data {
  id old;
  old = self->data;
  self->data = old ? [old stringByAppendingString:_data] : _data;
  [old release];
}

- (void)insertData:(NSString *)_data offset:(unsigned)_offset {
  [self doesNotRecognizeSelector:_cmd];
}

- (void)deleteData:(unsigned)_offset count:(unsigned)_count {
  NSRange r;
  id new, old;
  
  r.location = _offset;
  r.length   = _count;

  new = [self->data substringWithRange:r];
  old = self->data;
  self->data = [new copy];
  [old release];
}

- (void)replaceData:(unsigned)_offset count:(unsigned)_c with:(NSString *)_s {
  [self doesNotRecognizeSelector:_cmd];
}

/* parent node */

- (void)_domNodeRegisterParentNode:(id)_parent {
  self->parent = _parent;
}
- (void)_domNodeForgetParentNode:(id)_parent {
  if (_parent == self->parent)
    /* the node's parent was deallocated */
    self->parent = nil;
}
- (id)parentNode {
  return self->parent;
}

@end /* DOMCharacterData */

@implementation DOMCharacterData(QPValues)

- (NSException *)setQueryPathValue:(id)_value {
  [self setData:[_value stringValue]];
  return nil;
}
- (id)queryPathValue {
  return [self data];
}

@end /* DOMCharacterData(QPValues) */
