// $Id: DOMElement.h,v 1.2 2002/06/11 06:56:28 helge Exp $

#ifndef __DOMElement_H__
#define __DOMElement_H__

#include <DOM/DOMNode.h>

/*
  Why is there no removeAttributeNodeNS method?
  
  There is, but it's called removeAttributeNode. 

  We needed both setAttributeNode and setAttributeNodeNS, because those
  functions use different rules to select which (if any) existing Attr the
  new one will replace. setAttributeNode bases this decision on the nodeName,
  while setAttributeNodeNS looks at the combination of namespaceURI and
  localname. However, when you remove a specific Attr Node, its nodeName,
  localname, and namespaceURI are ignored, and there's no need for a second
  method to support this. 
*/

@class NSString, NSMutableDictionary, NSMutableArray;

@interface DOMElement : DOMNodeWithChildren < DOMElement >
{
  id                  parent;
  NSString            *tagName;
  NSMutableDictionary *keyToAttribute;
  NSMutableArray      *attributes;
  
  NSString *namespaceURI;
  NSString *prefix;

  /* positional info */
  unsigned line;
  
  /* caches */
  id attrNodeMap;
}

@end

@interface DOMElement(PrivateCtors)
/* use DOMDocument for constructing DOMElements ! */

- (id)initWithTagName:(NSString *)_tagName;
- (id)initWithTagName:(NSString *)_tagName namespaceURI:(NSString *)_uri;

@end

#endif /* __DOMElement_H__ */
