// $Id: SaxDeclHandler.h,v 1.1.1.1 2002/05/22 17:25:58 helge Exp $

#ifndef __SaxDeclHandler_H__
#define __SaxDeclHandler_H__

@class NSString;

/*
  new in SAX 2.0beta

  In Java this class is in the ext-package, that is,
  implementation is optional.

  SAX2 extension handler for DTD declaration events. 

  This is an optional extension handler for SAX2 to provide information
  about DTD declarations in an XML document. XML readers are not required
  to support this handler.

  Note that data-related DTD declarations (unparsed entities and notations)
  are already reported through the DTDHandler interface.

  If you are using the declaration handler together with a lexical handler,
  all of the events will occur between the startDTD and the endDTD events.

  To set the DeclHandler for an XML reader, use the setProperty method with
  the propertyId "http://xml.org/sax/handlers/DeclHandler". If the reader
  does not support declaration events, it will throw a
  SAXNotRecognizedException or a SAXNotSupportedException when you attempt
  to register the handler.
*/

@protocol SaxDeclHandler

/*
  Report an attribute type declaration.
  
  Only the effective (first) declaration for an attribute will be
  reported. The type will be one of the strings "CDATA", "ID", "IDREF",
  "IDREFS", "NMTOKEN", "NMTOKENS", "ENTITY", "ENTITIES", or "NOTATION",
  or a parenthesized token group with the separator "|" and all whitespace
  removed.

  valueDefault - A string representing the attribute default ("#IMPLIED",
  "#REQUIRED", or "#FIXED") or nil if none of these applies
*/
- (void)attributeDeclaration:(NSString *)_attributeName
  elementName:(NSString *)_elementName
  type:(NSString *)_type
  defaultType:(NSString *)_defType
  defaultValue:(NSString *)_defValue;

/*
  Report an attribute type declaration.
  
  Only the effective (first) declaration for an attribute will be
  reported. The type will be one of the strings "CDATA", "ID", "IDREF",
  "IDREFS", "NMTOKEN", "NMTOKENS", "ENTITY", "ENTITIES", or "NOTATION",
  or a parenthesized token group with the separator "|" and all whitespace
  removed.

  If it is a parameter entity, the name will begin with '%'.
*/
- (void)elementDeclaration:(NSString *)_name contentModel:(NSString *)_model;

/*
  Report a parsed external entity declaration.
  
  Only the effective (first) declaration for each entity will be reported.

  If it is a parameter entity, the name will begin with '%'.
*/
- (void)externalEntityDeclaration:(NSString *)_name
  publicId:(NSString *)_pub
  systemId:(NSString *)_sys;

/*
  Report an internal entity declaration.
  
  Only the effective (first) declaration for each entity will be reported.

  If it is a parameter entity, the name will begin with '%'.
*/
- (void)internalEntityDeclaration:(NSString *)_name value:(NSString *)_value;

@end

#endif /* __SaxDeclHandler_H__ */
