// $Id: SaxXMLReader.h,v 1.1.1.1 2002/05/22 17:25:58 helge Exp $

#ifndef __SaxXMLReader_H__
#define __SaxXMLReader_H__

#import <Foundation/NSObject.h>
#include <SaxObjC/SaxContentHandler.h>
#include <SaxObjC/SaxDTDHandler.h>
#include <SaxObjC/SaxErrorHandler.h>
#include <SaxObjC/SaxEntityResolver.h>

/*
  new in SAX 2.0beta, replaces SaxParser

  Interface for reading an XML document using callbacks. 

  This is a common interface that can be shared by many XML parsers. This
  interface allows an application to set and query features and properties
  in the parser, to register event handlers for document processing, and to
  initiate a document parse.

  This interface replaces the (now deprecated) SAX 1.0 Parser interface; it
  currently extends Parser to aid in the transition to SAX2, but it will
  likely not do so in any future versions of SAX.

  The Reader interface contains two important enhancements over the old
  Parser interface:

    1. it adds a standard way to query and set features and properties; and 
    2. it adds Namespace support, which is required for many higher-level XML
       standards. 

  There are adapters available to convert a SAX1 Parser to a SAX2 XMLReader
  and vice-versa.
*/

@protocol SaxXMLReader

/* features & properties */

- (void)setFeature:(NSString *)_name to:(BOOL)_value;
- (BOOL)feature:(NSString *)_name;
- (void)setProperty:(NSString *)_name to:(id)_value;
- (id)property:(NSString *)_name;

/* handlers */

- (void)setContentHandler:(id<NSObject,SaxContentHandler>)_handler;
- (void)setDTDHandler:(id<NSObject,SaxDTDHandler>)_handler;
- (void)setErrorHandler:(id<NSObject,SaxErrorHandler>)_handler;
- (void)setEntityResolver:(id<NSObject,SaxEntityResolver>)_handler;
- (id<NSObject,SaxContentHandler>)contentHandler;
- (id<NSObject,SaxDTDHandler>)dtdHandler;
- (id<NSObject,SaxErrorHandler>)errorHandler;
- (id<NSObject,SaxEntityResolver>)entityResolver;

/* parsing */

- (void)parseFromSource:(id)_source;
- (void)parseFromSource:(id)_source systemId:(NSString *)_sysId;
- (void)parseFromSystemId:(NSString *)_sysId;

@end

#endif /* __SaxXMLReader_H__ */
