// $Id: NSException+XmlRpcCoding.m,v 1.4 2002/07/03 08:39:03 helge Exp $

#include "XmlRpcMethodResponse.h"
#include "common.h"
#include "XmlRpcCoder.h"

@implementation NSException(XmlRpcCoding)

- (NSString *)xmlRpcType {
  return @"struct";
}

- (NSNumber *)xmlRpcFaultCode {
  NSDictionary *ui;
  id code;

  ui = [self userInfo];
  
  if ((code = [ui objectForKey:@"XmlRpcFaultCode"]))
    /* code is set */;
  else if ((code = [ui objectForKey:@"faultCode"]))
    /* code is set */;
  else
    code = [self name];
  
  return [NSNumber numberWithInt:[code intValue]];
}

- (void)encodeWithXmlRpcCoder:(XmlRpcEncoder *)_coder {
  int      code;
  NSString *str, *n, *r;
  NSDictionary *ui;
  
  code = [[self xmlRpcFaultCode] intValue];
  
  n = [self name];
  r = [self reason];
  
  if ([n length] == 0)
    str = r;
  else if ([r length] == 0)
    str = n;
  else
    str = [NSString stringWithFormat:@"%@: %@", n, r];
  
  if ((ui = [self userInfo]))
    str = [NSString stringWithFormat:@"%@ %@", str, ui];
  
  [_coder encodeInt:code   forKey:@"faultCode"];
  [_coder encodeString:str forKey:@"faultString"];
}

+ (NSString *)exceptionNameForXmlRpcFaultCode:(int)_code {
  return [NSString stringWithFormat:@"XmlRpcFault<%i>", _code];
}
- (NSString *)exceptionNameForXmlRpcFaultCode:(int)_code {
  return [[self class] exceptionNameForXmlRpcFaultCode:_code];
}

+ (id)decodeObjectWithXmlRpcCoder:(XmlRpcDecoder *)_coder {
  int          code;
  NSString     *r;
  NSDictionary *ui;
  
  code = [_coder decodeIntForKey:@"faultCode"];
  r    = [_coder decodeStringForKey:@"faultString"];
  ui   = [NSDictionary dictionaryWithObjectsAndKeys:
                         [NSNumber numberWithInt:code], @"faultCode",
                         r, @"faultString",
                         nil];
  
  return [self exceptionWithName:
                 [self exceptionNameForXmlRpcFaultCode:code]
               reason:r
               userInfo:ui];
}

@end /* NSException(XmlRpcCoding) */
