// $Id: XmlRpcEncoder.m,v 1.5 2002/12/27 09:57:40 helge Exp $

#include "XmlRpcCoder.h"
#include "XmlRpcMethodCall.h"
#include "XmlRpcMethodResponse.h"
#include "common.h"

@interface NSMutableString(XmlRpcDecoder)
- (void)appendHTMLString:(NSString *)_value;
@end

@interface NSData(UsedNGExtensions)
- (NSData *)dataByEncodingBase64;
@end

@implementation XmlRpcEncoder

- (id)initForWritingWithMutableString:(NSMutableString *)_string {
  if ((self = [super init])) {
    self->string = [_string retain];
    self->timeZone = [[NSTimeZone timeZoneWithAbbreviation:@"GMT"] retain];
  }
  return self;
}

- (void)dealloc {
  [self->string release];
  
  [self->objectStack release];
  [self->objectHasStructStack release];

  [self->timeZone release];
  [super dealloc];
}

- (BOOL)profileXmlRpcCoding {
  return [[NSUserDefaults standardUserDefaults]
                          boolForKey:@"ProfileXmlRpcCoding"];
}

/* accessors */

- (void)setDefaultTimeZone:(NSTimeZone *)_timeZone {
  [self->timeZone autorelease];
  self->timeZone = [_timeZone retain];
}

/*"
  This returns the timezone which will be associated with XML-RPC
  datetime objects. Note: XML-RPC datetime values have no! associated timezone,
  it's recommended that you always use UTC though.
"*/
- (NSTimeZone *)defaultTimeZone {
  return self->timeZone;
}

/* *** */

- (void)encodeMethodCall:(XmlRpcMethodCall *)_methodCall {
  NSEnumerator *paramEnum = [[_methodCall parameters] objectEnumerator];
  id           param;
  NSTimeInterval st     = 0.0;

  if ([self profileXmlRpcCoding])
    st = [[NSDate date] timeIntervalSince1970];
  
  [self->string appendString:@"<?xml version='1.0'?>"];
  [self->string appendString:@"<methodCall>"];
  
  [self->string appendString:@"<methodName>"];
  [self->string appendString:[_methodCall methodName]];
  [self->string appendString:@"</methodName>"];
  
  [self->string appendString:@"<params>"];
  
  while ((param = [paramEnum nextObject])) {
    [self->string appendString:@"<param>"];
    [self->string appendString:@"<value>"];

    [self encodeObject:param];
    
    [self->string appendString:@"</value>"];
    [self->string appendString:@"</param>"];
  }

  [self->string appendString:@"</params>"];
  [self->string appendString:@"</methodCall>"];

  if ([self profileXmlRpcCoding]) {
    NSTimeInterval diff;
    diff = [[NSDate date] timeIntervalSince1970] - st;
    
    printf("+++     encodeMethodCall: %0.5fs\n", diff);
  }
}

- (void)encodeMethodResponse:(XmlRpcMethodResponse *)_methodResponse {
  id result = [_methodResponse result];
  static Class ExceptionClass = Nil;
  NSTimeInterval st           = 0.0;

  if ([self profileXmlRpcCoding])
    st = [[NSDate date] timeIntervalSince1970];

  if (ExceptionClass == Nil)
    ExceptionClass = [NSException class];

  [self->string appendString:@"<?xml version='1.0'?>"];
  [self->string appendString:@"<methodResponse>"];

  if ([result isKindOfClass:ExceptionClass]) {
    [self->string appendString:@"<fault>"];
    [self->string appendString:@"<value>"];

    [self encodeObject:result];
    
    [self->string appendString:@"</value>"];
    [self->string appendString:@"</fault>"];
  }
  else {
    [self->string appendString:@"<params>"];
    [self->string appendString:@"<param>"];
    [self->string appendString:@"<value>"];

    [self encodeObject:result];
    
    [self->string appendString:@"</value>"];
    [self->string appendString:@"</param>"];
    [self->string appendString:@"</params>"];
  }

  [self->string appendString:@"</methodResponse>"];

  if ([self profileXmlRpcCoding]) {
    NSTimeInterval diff;
    diff = [[NSDate date] timeIntervalSince1970] - st;
    
    printf("+++ enocdeMethodResponse: %0.5fs\n", diff);
  }
}


- (void)_reset {
  [self->objectStack release];          self->objectStack          = nil;
  [self->objectHasStructStack release]; self->objectHasStructStack = nil;
}

- (void)_ensureStacks {
  if (self->objectStack == nil)
    self->objectStack = [[NSMutableArray alloc] initWithCapacity:8];
  if (self->objectHasStructStack == nil)
    self->objectHasStructStack = [[NSMutableArray alloc] initWithCapacity:8];
}

- (void)_encodeObject:(id)_object {
  if (_object) {
    [self _ensureStacks];
    [self->objectStack addObject:_object];
    [self->objectHasStructStack addObject:@"NO"];

    [_object encodeWithXmlRpcCoder:self];

    if ([[self->objectHasStructStack lastObject] boolValue]) {
      [self->string appendString:@"</struct>"];
    }
    [self->objectHasStructStack removeLastObject];
    [self->objectStack removeLastObject];
  }
}

- (NSString *)_className {
  id obj;
  
  if ((obj = [self->objectStack lastObject]) == nil)
    return nil;

  if ([obj isKindOfClass:[NSString class]])
    return @"NSString";
  
  return NSStringFromClass([obj classForCoder]);
}

- (void)encodeObject:(id)_obj {
  [self _encodeObject:_obj];
}


- (void)_appendTagName:(NSString *)_tagName attributes:(NSDictionary *)_attrs {
  NSEnumerator *keyEnum   = [_attrs keyEnumerator];
  NSString     *key       = nil;
//  NSString     *className = [self _className];
  
  [self->string appendString:@"<"];
  [self->string appendString:_tagName];
  
#if 0
#  warning adding of NSObjectClass removed due to compatibility issues !
  if ([className length] > 0) {
    [self->string appendString:@" NSObjectClass=\""];
    [self->string appendHTMLString:className];
    [self->string appendString:@"\""];
  }
#endif

  while ((key = [keyEnum nextObject])) {
    [self->string appendString:@" "];
    [self->string appendString:key];
    [self->string appendString:@"=\""];
    [self->string appendHTMLString:[_attrs objectForKey:key]];
    [self->string appendString:@"\""];
  }
  [self->string appendString:@">"];  
}

- (void)_appendTagName:(NSString *)_tagName {
  [self _appendTagName:_tagName attributes:nil];
}

- (void)_encodeNumber:(NSNumber *)_number
              tagName:(NSString *)_tagName
{
  [self _appendTagName:_tagName];
  
  if ([_tagName isEqualToString:@"boolean"])
    [self->string appendString:[_number boolValue] ? @"1" : @"0"];
  else
    [self->string appendHTMLString:[_number stringValue]];
  
  [self->string appendString:@"</"];
  [self->string appendString:_tagName];
  [self->string appendString:@">"];
  
}

- (void)encodeStruct:(NSDictionary *)_struct {
  NSEnumerator *keys = nil;
  id           key   = nil;
  
  [self _appendTagName:@"struct"];
  
  keys = [_struct keyEnumerator];
  while ((key = [keys nextObject])) {
    [self->string appendString:@"<member>"];
    
    [self->string appendString:@"<name>"];
    [self->string appendHTMLString:key];
    [self->string appendString:@"</name>"];

    [self->string appendString:@"<value>"];

    [self _encodeObject:[_struct objectForKey:key]];
    
    [self->string appendString:@"</value>"];
    
    [self->string appendString:@"</member>"];
  }
  
  [self->string appendString:@"</struct>"];
}

- (void)encodeArray:(NSArray *)_array {
  NSEnumerator    *valueEnum = [_array objectEnumerator];
  id value;

  [self _appendTagName:@"array"];

  [self->string appendString:@"<data>"];
  
  while ((value = [valueEnum nextObject])) {
    [self->string appendString:@"<value>"];

    [self _encodeObject:value];
    
    [self->string appendString:@"</value>"];
  }
  
  [self->string appendString:@"</data>"];
  [self->string appendString:@"</array>"];
}

- (void)encodeBase64:(NSData *)_data {
  NSString *base64;
  
  base64 = [[NSString alloc] initWithData:[_data dataByEncodingBase64]
                             encoding:NSASCIIStringEncoding];

  [self _appendTagName:@"base64"];
  [self->string appendString:base64];
  [self->string appendString:@"</base64>"];

  [base64 release]; base64 = nil;
}

- (void)encodeBoolean:(BOOL)_number {
  [self _encodeNumber:[NSNumber numberWithBool:_number] tagName:@"boolean"];
}

- (void)encodeInt:(int)_number {
  [self _encodeNumber:[NSNumber numberWithInt:_number] tagName:@"int"];
}

- (void)encodeDouble:(double)_number {
  [self _encodeNumber:[NSNumber numberWithDouble:_number] tagName:@"double"];
}

- (void)encodeString:(NSString *)_string {
  [self _appendTagName:@"string"];
  [self->string appendHTMLString:_string];
  [self->string appendString:@"</string>"];
}

- (void)encodeDateTime:(NSDate *)_date {
  static NSTimeZone *gmt = nil;
  NSDictionary *attrs;
  NSString     *s;
  NSTimeZone   *tz;
  
  if (gmt == nil)
    gmt = [[NSTimeZone timeZoneWithAbbreviation:@"GMT"] retain];
  
  s = [_date descriptionWithCalendarFormat:@"%Y%m%dT%H:%M:%S"
             timeZone:gmt
             locale:nil];
  
  if ([_date respondsToSelector:@selector(timeZone)]) {
    if ((tz = [(NSCalendarDate *)_date timeZone]) == nil)
      tz = self->timeZone;
  }
  else
    tz = self->timeZone;
  
  attrs = [NSDictionary dictionaryWithObjectsAndKeys:
                          [tz abbreviation], @"timeZone", nil];
  
  [self _appendTagName:@"dateTime.iso8601" attributes:attrs];
  [self->string appendString:s];
  [self->string appendString:@"</dateTime.iso8601>"];
}

- (void)_appendMember:(id)_obj forKey:(NSString *)_key selector:(SEL)_sel {
  [self _ensureStacks];
  
  if (![[self->objectHasStructStack lastObject] boolValue]) {
    [self->objectHasStructStack removeLastObject];
    [self->objectHasStructStack addObject:@"YES"];
    [self _appendTagName:@"struct"];
  }  
  if (_obj != nil) {
    [self->objectStack addObject:_obj];
    [self->string appendString:@"<member><name>"];
    [self->string appendString:_key];
    [self->string appendString:@"</name><value>"];
    [self performSelector:_sel withObject:_obj];
    [self->string appendString:@"</value></member>"];
    [self->objectStack removeLastObject];
  }
}

- (void)encodeStruct:(NSDictionary *)_struct forKey:(NSString *)_key {
  [self _appendMember:_struct forKey:_key selector:@selector(encodeStruct:)];
}
- (void)encodeArray:(NSArray *)_array  forKey:(NSString *)_key {
  [self _appendMember:_array forKey:_key selector:@selector(encodeArray:)];
}
- (void)encodeBase64:(NSData *)_data forKey:(NSString *)_key {
  [self _appendMember:_data forKey:_key selector:@selector(encodeBase64:)];
}
- (void)encodeBoolean:(BOOL)_number forKey:(NSString *)_key {
  [self _appendMember:[NSNumber numberWithBool:_number]
        forKey:_key selector:@selector(encodeBoolean:)];
}
- (void)encodeInt:(int)_number forKey:(NSString *)_key {
  [self _appendMember:[NSNumber numberWithInt:_number]
        forKey:_key selector:@selector(encodeInt:)];
}
- (void)encodeDouble:(double)_number forKey:(NSString *)_key {
  [self _appendMember:[NSNumber numberWithDouble:_number]
        forKey:_key selector:@selector(encodeDouble:)];
}
- (void)encodeString:(NSString *)_string forKey:(NSString *)_key {
  [self _appendMember:_string forKey:_key selector:@selector(encodeString:)];
}
- (void)encodeDateTime:(NSDate *)_date forKey:(NSString *)_key {
  [self _appendMember:_date forKey:_key selector:@selector(encodeDateTime:)];
}
- (void)encodeObject:(id)_object forKey:(NSString *)_key {
  [self _appendMember:_object forKey:_key selector:@selector(encodeObject:)];
}

@end /* XmlRpcEncoder */

@implementation NSMutableString(XmlRpcDecoder)

- (void)appendHTMLString:(NSString *)_value {
#if 1 // TODO: to be tested !
  unsigned i, j, len;
  BOOL     didEscape = NO;
  unichar  *buf, *escbuf = NULL;
  
  if ((len = [_value length]) == 0)
    /* nothing to add ... */
    return;
    
  if (len == 1) {
    /* a single char */
    unichar c;
    
    switch ((c = [_value characterAtIndex:0])) {
      case '&': [self appendString:@"&amp;"];  break;
      case '<': [self appendString:@"&lt;"];   break;
      case '>': [self appendString:@"&gt;"];   break;
      case '"': [self appendString:@"&quot;"]; break;
      default:  [self appendString:_value];    break;
    }
    return;
  }
  
  buf = calloc(len + 2, sizeof(unichar));
  [_value getCharacters:buf];
  
  for (i = 0, j = 0; i < len; i++) {
    switch (buf[i]) {
      case '&':
      case '<':
      case '>':
      case '"': {
        didEscape = YES;
        if (escbuf == NULL) {
          /* worst case: string consists of quotes */
          escbuf = calloc(len * 6 + 2, sizeof(unichar));
          memcpy(escbuf, buf, i * sizeof(unichar));
          j = i;
        }
        escbuf[j++] = '&';
        switch (buf[i]) {
          case '&':
            escbuf[j++] = 'a'; escbuf[j++] = 'm'; escbuf[j++] = 'p';
            break;
          case '<':
            escbuf[j++] = 'l'; escbuf[j++] = 't';
            break;
          case '>':
            escbuf[j++] = 'g'; escbuf[j++] = 't';
            break;
          case '"':
            escbuf[j++] = 'q'; escbuf[j++] = 'u';
            escbuf[j++] = 'o'; escbuf[j++] = 't';
            break;
        }
        escbuf[j++] = ';';
        break;
      }
      default:
        if (escbuf)
          escbuf[j++] = buf[i];
        break;
    }
  }
  
  if (escbuf) {
    NSString *s;
    
    s = [[NSString alloc] initWithCharacters:escbuf length:j];
    [self appendString:s];
    [s release];
    free(escbuf);
  }
  else
    [self appendString:_value];
  
  if (buf) free(buf);
#else
#warning UNICODE !!!
  void (*addBytes)(id,SEL,const char *,unsigned);
  id            dummy = nil;
  unsigned      clen;
  unsigned char *cbuf;
  unsigned char *cstr;

  if ([_value length] == 0) return;

  clen = [_value cStringLength];
  if (clen == 0) return; /* nothing to add .. */
  
  cbuf = cstr = malloc(clen + 1);
  [_value getCString:cbuf]; cbuf[clen] = '\0';
  dummy = [[NSMutableData alloc] initWithCapacity:2*clen];

  addBytes = (void*)[dummy methodForSelector:@selector(appendBytes:length:)];
  NSAssert(addBytes != NULL, @"could not get appendBytes:length: ..");
  
  while (*cstr) {
    switch (*cstr) {
      case '&':
        addBytes(dummy, @selector(appendBytes:length:), "&amp;", 5);
        break;
      case '<':
        addBytes(dummy, @selector(appendBytes:length:), "&lt;", 4);
        break;
      case '>':
        addBytes(dummy, @selector(appendBytes:length:), "&gt;", 4);
        break;
      case '"':
        addBytes(dummy, @selector(appendBytes:length:), "&quot;", 6);
        break;
      
      default:
        addBytes(dummy, @selector(appendBytes:length:), cstr, 1);
        break;
    }
    cstr++;
  }
  free(cbuf);
  {
    NSString *tmp = nil;
    
    tmp = [[NSString alloc] initWithData:dummy
                            encoding:[NSString defaultCStringEncoding]];
    [self appendString:tmp];
    [tmp release];
  }
#endif
}

@end /* NSMutableString(XmlRpcDecoder) */
