// $Id: XmlRpcMethodCall.m,v 1.2 2002/09/30 10:44:05 helge Exp $

#include <XmlRpc/XmlRpcMethodCall.h>
#include <XmlRpc/XmlRpcCoder.h>
#include "common.h"

@interface XmlRpcMethodCall(PrivateMethodes)
- (NSString *)_encodeXmlRpcMethodCall;
@end

@interface NSArray(UsedEOControl)
- (id)shallowCopy;
@end

@implementation XmlRpcMethodCall

- (id)initWithXmlRpcString:(NSString *)_string {
  XmlRpcDecoder    *coder;
  XmlRpcMethodCall *baseCall;
  
  if ([_string length] == 0) {
    [self release];
    return nil;
  }
  
  coder = [[XmlRpcDecoder alloc] initForReadingWithString:_string];
  baseCall = [coder decodeMethodCall];
  [coder release];

  if (baseCall == nil) {
    [self release];
    return nil;
  }
  
  self = [self initWithMethodName:[baseCall methodName]
               parameters:[baseCall parameters]];
  
  return self;
}
- (id)initWithXmlRpcData:(NSData *)_xmlRpcData {
  XmlRpcDecoder    *coder;
  XmlRpcMethodCall *baseCall;
  
  if ([_xmlRpcData length] == 0) {
    [self release];
    return nil;
  }
  
  coder = [[XmlRpcDecoder alloc] initForReadingWithData:_xmlRpcData];
  baseCall = [coder decodeMethodCall];
  [coder release];
  
  if (baseCall == nil) {
    [self release];
    return nil;
  }
  
  self = [self initWithMethodName:[baseCall methodName]
               parameters:[baseCall parameters]];
  
  return self;
}

- (id)initWithMethodName:(NSString *)_name parameters:(NSArray *)_params {
  if ((self = [super init])) {
    self->methodName = [_name copy];
    self->parameters = [_params shallowCopy];
  }
  return self;
}

- (void)dealloc {
  [self->methodName release];
  [self->parameters release];
  [super dealloc];
}

/* accessors */

- (void)setMethodName:(NSString *)_name {
  [self->methodName autorelease];
  self->methodName = [_name copy];
}
- (NSString *)methodName {
  return self->methodName;
}

- (void)setParameters:(NSArray *)_params {
  id tmp = self->parameters;
  self->parameters = [_params shallowCopy];
  [tmp release];
}
- (NSArray *)parameters {
  return self->parameters;
}

- (NSString *)xmlRpcString {
  return [self _encodeXmlRpcMethodCall];
}

/* description */

- (NSString *)description {
  NSMutableString *s;
  
  s = [NSMutableString stringWithFormat:@"<0x%08X[%@]: ",
                         self, NSStringFromClass([self class])];
  [s appendFormat:@"method=%@", [self methodName]];
  [s appendFormat:@" #paras=%d", [self->parameters count]];
  [s appendString:@">"];
  return s;
}

@end /* XmlRpcMethodCall */


@implementation XmlRpcMethodCall(PrivateMethodes)

- (NSString *)_encodeXmlRpcMethodCall {
  NSMutableString *str;
  XmlRpcEncoder   *coder;

#if DEBUG
  NSAssert1(self->methodName, @"%s, methodName is not allowed to be nil!",
            __PRETTY_FUNCTION__);
#endif
  
  str   = [NSMutableString stringWithCapacity:512];
  coder = [[XmlRpcEncoder alloc] initForWritingWithMutableString:str];
  [coder encodeMethodCall:self];

  [coder release]; coder = nil;
  
  return str;
}

@end /* XmlRpcMethodCall(PrivateMethodes) */
