// $Id: XmlRpcRequestDecoder.m,v 1.2 2002/07/01 10:04:18 helge Exp $

#include <XmlRpc/XmlRpcCoder.h>
#include <XmlRpc/XmlRpcSaxHandler.h>
#include <XmlRpc/XmlRpcMethodCall.h>
#include <XmlRpc/XmlRpcValue.h>
#include <SaxObjC/SaxXMLReaderFactory.h>
#include "common.h"

@implementation XmlRpcRequestDecoder

static id saxRequestHandler = nil;
static id requestParser     = nil;

- (void)_ensureSaxAndParser {
  if (saxRequestHandler == nil) {
    static Class clazz = Nil;
    
    if (clazz == Nil)
      clazz = NSClassFromString(@"XmlRpcSaxRequestHandler");
    
    if ((saxRequestHandler = [[clazz alloc] init]) == nil) {
      NSLog(@"%s: did not find sax handler ...", __PRETTY_FUNCTION__);
      return;
    }
  }
  
  if (requestParser != nil) return;
  
  requestParser =
    [[SaxXMLReaderFactory standardXMLReaderFactory] createXMLReader];
  
  if (requestParser == nil) {
    NSLog(@"%s: did not find an XML parser ...", __PRETTY_FUNCTION__);
    return;
  }
  
  [requestParser setContentHandler:saxRequestHandler];
  [requestParser setDTDHandler:saxRequestHandler];
  [requestParser setErrorHandler:saxRequestHandler];

  [requestParser retain];
}

- (XmlRpcMethodCall *)decodeRootObject {
  XmlRpcMethodCall *methodCall;
  NSEnumerator     *paramEnum;
  NSMutableArray   *params;
  XmlRpcValue      *param;
  
  [self _ensureSaxAndParser];
  
  [saxRequestHandler reset];
  
  [requestParser parseFromSource:self->string systemId:nil];
  
  methodCall = [saxRequestHandler methodCall];
  
  // the methodCall's parameters is an array of XmlRpcValues!!!
  
  paramEnum = [[methodCall parameters] objectEnumerator];
  params    = [[NSMutableArray alloc] initWithCapacity:
                                      [[methodCall parameters] count]];

  while ((param = [paramEnum nextObject])) {
    Class objClass = Nil;
    id    obj;
    
    [self->value autorelease];
    self->value = [param retain];
    
    if ((objClass = NSClassFromString([param className])) != Nil) {
      if ((obj = [objClass decodeObjectWithXmlRpcCoder:self])) {
        [params addObject:obj];
      }
      else {
        NSLog(@"%s: Warning: try to add 'nil' object to params (class='%@')",
              __PRETTY_FUNCTION__,
              [param className]);
      }
    }
  }
  [methodCall setParameters:params];
  
  [params release];

  return methodCall;
}

@end /* XmlRpcRequestDecoder */
