// $Id: XmlRpcValue.m,v 1.3 2003/01/02 09:53:00 helge Exp $

#include "XmlRpcValue.h"
#include "common.h"

@implementation XmlRpcValue
/*"
  The XmlRpcValue class is used internally by the XML-RPC decoder to
  represent any valid XML-RPC value. You should never need to use this class.
"*/

- (id)initWithValue:(id)_value className:(NSString *)_className {
  if ((self = [super init])) {
    NSString *cName;
    
    ASSIGN(self->value, _value);
    cName = (_className != nil)
      ? _className
      : NSStringFromClass([_value class]);

    ASSIGN(self->className, cName);
  }
  return self;
}

- (void)dealloc {
  [self->className release];
  [self->value     release];
  [super dealloc];
}

- (id)value {
  return self->value;
}

- (void)setClassName:(NSString *)_className {
  if (_className != self->className) {
    [self->className autorelease];
    self->className = [_className copy];
  }
}
- (NSString *)className {
  return self->className;
}

- (Class)class {
  return NSClassFromString([self className]);
}

- (BOOL)isException {
  return [[self class] isKindOfClass:[NSException class]];
}
- (BOOL)isDictionary {
  return [[self class] isKindOfClass:[NSDictionary class]];
}

/* description */

- (NSString *)description {
  return [NSString stringWithFormat:@"XmlRpcValue: %@->%@",
                   self->className,
                   self->value];
}

/* NSCopying */

- (id)copyWithZone:(NSZone *)zone {
  return [[[self class] alloc]
                 initWithValue:self->value className:self->className];
}

@end /* XmlRpcValue */
