// $Id: NSString+ICal.m,v 1.1 2002/10/10 08:06:26 helge Exp $

#include "NSString+ICal.h"
#include "common.h"
#include <ical.h>

@implementation NSString(ICalCString)

- (const char *)icalCString {
  NSStringEncoding enc;
  
  enc = [[self class] libicalStringEncoding];
  
  if (enc == NSUTF8StringEncoding)
    return [self UTF8String];
  else if (enc == [NSString defaultCStringEncoding])
    return [self cString];
  else
    return [[self dataUsingEncoding:enc] bytes];
}

- (NSString *)icalString {
  return self;
}

@end /* NSString(ICalCString) */

@implementation NSObject(TemporaryStringInit)

+ (NSStringEncoding)libicalStringEncoding {
  return NSUTF8StringEncoding;
}

- (id)initWithICalCString:(const char *)_cstr {
  NSStringEncoding enc;

  if (_cstr == NULL) {
    RELEASE(self);
    return nil;
  }
  
  enc = [[self class] libicalStringEncoding];
  
  if (enc == NSUTF8StringEncoding)
    return [(NSString *)self initWithUTF8String:_cstr];
  else if (enc == [[self class] defaultCStringEncoding])
    return [(NSString *)self initWithCString:_cstr];
  else {
    NSData *d;
    
    d = [[NSData alloc] initWithBytes:_cstr length:strlen(_cstr)];
    self = [(NSString *)self initWithData:d encoding:enc];
    RELEASE(d);
    
    return self;
  }
  
  return nil;
}

- (id)initWithICalValueHandle:(icalvalue *)_handle {
  const char *s;
  
  if (_handle == NULL) {
    RELEASE(self);
    return nil;
  }
  if ((s = icalvalue_as_ical_string(_handle)) == NULL) {
    RELEASE(self);
    return nil;
  }
  return [self initWithICalCString:s];
}

- (id)initWithICalValueOfProperty:(icalproperty *)_prop {
  icalvalue *val;

  if (_prop == nil) {
    RELEASE(self);
    return nil;
  }
  
  if ((val = icalproperty_get_value(_prop)) == NULL) {
    NSLog(@"%s: ical property has no value ??", __PRETTY_FUNCTION__);
    RELEASE(self);
    return nil;
  }
  
  return [self initWithICalValueHandle:val];
}

@end /* NSObject(TemporaryStringInit) */

